# --------------------------
# Utils folder
# Class definition
#
# Lang : lang
# --------------------------

""" imports: logging, config and language libs """
import logging
import logging.config
from biaspeech.utils.config import Config

""" objects:  """
conf = Config() # config

if conf.OS == "macos":
    import argostranslate.package
    import argostranslate.translate
    from py3langid.langid import LanguageIdentifier, MODEL_FILE 

class Lang:

    """ constructor """
    def __init__(self): 
        self.logger = logging.getLogger(__name__) # logger
    
    """ translate """
    def get_translate(self, text, source, dest):
                        
        result = text
        if source == dest or conf.OS != "macos":
            result = text
        else:
            result = argostranslate.translate.translate(text, source, dest)
        
        result = result.lower()
        
        return result
    
    """ get_lang """
    def get_lang(self, text):
        try:
            result = "fr"
            if conf.OS == "macos":
                identifier = LanguageIdentifier.from_pickled_model(MODEL_FILE, norm_probs=True) 
                result = identifier.classify(line)
                
        except Exception as err:
            result = "fr"

        return result
        



# --------------------------
# Controller folder
# Class definition
#
# Controller : define the base controller
# --------------------------

""" imports: logging and config """
from biaspeech.utils.config import Config
import logging
import logging.config

""" imports: MVC """
from biaspeech.utils.paraphrase import Paraphrase
from biaspeech.utils.config import Config
from biaspeech.utils.speech2Text import Speech2Text
from biaspeech.utils.text2Speech import Text2Speech
from biaspeech.models.openAI.ai import AI
from biaspeech.models.robot.robot import Bia
from biaspeech.models.speech.prompt import Prompt
from biaspeech.views.ui import UI
from biaspeech.views.app import App

""" objects: logging and config """
conf = Config()  # config


class Controller:
    
    """ constructor """
    def __init__(self):

        """ util objects """
        self.logger = logging.getLogger(__name__)  # logger
        
        """ attributes """   
        self.ai = AI()  # object for openAI
        
    """ start_server """      
    def start_server(self): 
        old_text_in = ""
        if conf.UI == "keyboard":  # keyboard
            self.ui_view = UI()  # view object, ui
            while True:
                text_in = self.ui_view.get_prompt("? ")  # text prompt
                if text_in != old_text_in and text_in != "":  # something changed
                    self.process(text_in)  # process                     
                    old_text_in = text_in  # update

        elif conf.UI == "voice": # voice
            self.speech2Text = Speech2Text(conf.LANGUAGE) # speech2Text  
            while True:
                self.speech2Text.do_listen() # start listening
                text_in = self.speech2Text.get_input().lower() # STT input 
                if text_in != '' and text_in != old_text_in:
                    self.process(text_in)  # process                     
                    old_text_in = text_in  # update

        else:  # gui
            self.app_view = App()  # view object, app
            self.app_view.mainloop()  # open the app            
    
    """ process """   
    def process(self, text_in):
        prompt = Prompt(text_in)  # prompt object  
        bia = Bia(prompt, self.ai)  # bia object
                
        bia.run()  # run the action if any    
        output = bia.get_output()  # get the output
        
        self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech            
        self.text2Speech.do_speech(output)  # inform the user
            
        bia.save()  # save to sqlite

        return output
    
    """ help """   
    def help(self): 
        print(conf.HELP)  # help
        
    """ update """   
    def update(self): 
        self.paraphrase = Paraphrase()  # paraphrase              
        self.paraphrase.set_paraphrase()  # update the paraphrase table
        
    """ version """   
    def version(self): 
        print(conf.VERSION)  # version




# ---------
# BIA program
# - --
# IA personal assistant. BIA is OpenAI based vocal assistant. 
# Extra cool functionality, BIA also provides IA to the educational Otto robot.
#
# Nicolas Christophe (nicolas.christophe@gmail.com)
#
# Versions
#  - v1.0 15.04.2023 Creation
# ---------

""" imports: logging, config and language libs """
import logging
import logging.config
from biaspeech.utils import config
import sys

""" imports: MVC """
from biaspeech.controllers.base import Controller
from biaspeech.utils.config import Config

""" main """
def biaspeech():       
    """ controller:  """
    logger = logging.getLogger(__name__) # logger
            
    controller = Controller() # object for controller    
    if len(sys.argv) == 1: # no argument given
        controller.start_server() # starts the server, interactive mode
        
    else:
        if sys.argv[1] == "-help":
            controller.help()
        elif sys.argv[1] == "-update":
            controller.update()
        elif sys.argv[1] == "-version":
            controller.version()
        else:
            controller.process(sys.argv[1]) # unit mode, process only one input
    
 
if __name__ == "__main__":
    biaspeech() # let's go 
    



# --------------------------
# Models folder
# Class definition
#
# Prompt : define the prompt model
# --------------------------

""" imports: logging and config """
import logging
import logging.config
from biaspeech.utils.config import Config
        
""" objects:  """
conf = Config() # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
import nltk
from nltk import sent_tokenize
from nltk.sentiment import SentimentIntensityAnalyzer
from biaspeech.models.database.mia import Mia
from biaspeech.models.database.mia_utils import Mia_utils
from biaspeech.utils.lang import Lang

class Prompt:
    
    """ constructor """
    def __init__(self, input):     
            
        """ util objects """
        self.conf = Config() # config
        self.logger = logging.getLogger(__name__) # logger
        self.lang_utils = Lang() # lang

        """ attributes """                
        self.input = input.lower() # input
        self.lang = self.lang_utils.get_lang(self.input) # detect the lang
        self.input_en = "" # input_en
        self.set_lang(); # set the input_en
        self.weights = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0] 
        
        self.has_mom = False # mom found in the cache
        self.has_grandma = False # grandma found in the cache
        
        """ initialize """
        self.set_weight_words() # set the weight for each type, example question=100, etc
        self.set_weight_sentiment() # set the 3 weights positive, negative, neutral   
        self.set_type() # set the type, like question, etc
        self.set_mia() # create the ia object (= database table entry line object)
        self.set_cache() # cache management, do we have a mom or grandma relevant answer?
        
        """ inform the user """
        self.logger.info("input : " + self.input) # inform the user  
        self.logger.info("input_en : " + self.input_en) # inform the user  
        self.logger.info("weights : " + str(self.weights)) # inform the user
        self.logger.info("type : " + str(self.type)) # inform the user

    """ set_lang """         
    def set_lang(self):
        self.mia_utils = Mia_utils()
        res = self.mia_utils.get_input_en(self.input, self.lang)

        if len(res) > 0: # search a translation in mia if any
            self.input_en = ''.join(res[0])

        if self.input_en == "":  
            self.input_en = self.lang_utils.get_translate(self.input, self.lang, self.conf.LANG_TO) # translate to EN, pivot language
        

    """ set_weight words """         
    def set_weight_words(self):        
        from nltk.tokenize import word_tokenize
        question = self.input_en
        question = question.lower()
        question = word_tokenize(question)
        
        i = 0
        while i < len(self.weights):   
            if any(x in question[0] for x in self.conf.WORDS[i]):
                self.weights[i] = 100       
            i = i + 1 
                            
    """ set_weight sentiment """        
    def set_weight_sentiment(self):
        sia = SentimentIntensityAnalyzer()
        vs = sia.polarity_scores(self.input_en)

        self.weights.append(vs[self.conf.POSITIVE] * 100)
        self.weights.append(vs[self.conf.NEGATIVE] * 100)
        self.weights.append(vs[self.conf.NEUTRAL] * 100)

    """ set_type """        
    def set_type(self):
        self.type = "question"
        i = 0
        while i < len(self.weights):               
            if self.weights[i] > self.conf.WEIGHT_AVG:
                self.type = self.conf.TYPES[i]
                return
            i = i + 1 
         
    """ set_mia """
    def set_mia(self):
        self.mia = Mia.create(0, self.type, self.lang, self.input, self.input_en, "", "", "",
            self.weights[0], self.weights[1], self.weights[2], self.weights[3], self.weights[4], self.weights[5], 
            self.weights[6], self.weights[7], self.weights[8], self.weights[9], self.weights[10], self.weights[11], 
            self.weights[12], self.weights[13], self.weights[14], 0, "")  

    """ set_cache """
    def set_cache(self): 
        if self.type in self.conf.CACHED: # if the type is cache relevant
            self.set_mom() # mom object
            self.set_grandma() # grandma object
            self.set_update_mia() # if cache relevant copy cache to mia
                       
    """ set_mom """
    def set_mom(self):          
        if len(self.mia.get_mom(self.input_en)) > 0:
            self.mom = self.mia.get_mom(self.input_en)[0] # search for a parent in the mia table
            self.has_mom = True # set the attribute          
            
    """ set_grandma """
    def set_grandma(self):
        if len(self.mia.get_grandma(self.input_en)) > 0:
            self.grandma = self.mia.get_grandma(self.input_en)[0] # search for parent in the grandma table
            self.has_grandma = True # set the attribute
                            
    """ set_update_mia """
    def set_update_mia(self):
        if self.has_mom: # mom found
            w_positive_mom = self.mom.w_positive # get the weight of the mom
            w_neutral_mom = self.mom.w_neutral # get the weight of the mom
           
            if w_positive_mom >= self.conf.WEIGHT_MIN or self.type in conf.FORCE_CACHED or w_neutral_mom == 100: # if valid cache        
                self.mia = self.mom 
                return
               
        if self.has_grandma: # grandma found
            w_positive_grandma = self.grandma.w_positive # get the weight of the grandma
            w_neutral_grandma = self.grandma.w_neutral # get the weight of the grandma
                    
            if w_positive_grandma >= self.conf.WEIGHT_MIN or self.type in conf.FORCE_CACHED or w_neutral_grandma == 100: # if trigger reached
                self.mia = self.grandma # grandma cache found is valid, set it



# --------------------------
# Views folder
# Class definition
#
# Ui : define the base UI view (keyboard and voice, no gui)
# --------------------------

class UI:

    """ constructor """
    def __init__(self):
        pass

    """ get_prompt """
    def get_prompt(self, prompt):    
        text = input(prompt) # ask the user to prompt
        return text

        



# --------------------------
# Models folder
# Class definition
#
# Default : define the Default (Block) model
# --------------------------

""" imports: logging and config """
import logging
import logging.config
from biaspeech.utils.config import Config

""" objects:  """
conf = Config()  # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
from biaspeech.models.blocks import block


class Default(block.Block):
    
    """ constructor """
    def __init__(self, prompt, ai): 
        block.Block.__init__(self, prompt, ai)
        
    """ get_output """        
    def get_output(self): 
        self.logger.info("asking OpenAI")  # inform the user
          
        return(self.ai.get_output(self.prompt.input))  # ask OpenAI
    
    """ get_file """        
    def get_file(self):
        pass
              
    """ run """        
    def run(self, file):
        pass



# --------------------------
# Models folder
# Class definition
#
# Camera : define the Camera (Block) model
# --------------------------

""" imports: logging and config, datetime, time, opencv """
import logging
import logging.config
from biaspeech.utils.config import Config
import time
from datetime import datetime
import cv2

""" objects:  """
conf = Config()  # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
from biaspeech.models.blocks import block


class Camera(block.Block):
    
    """ constructor """
    def __init__(self, prompt, ai): 
        block.Block.__init__(self, prompt, ai)
        
    """ get_output """        
    def get_output(self): 
        return(conf.OUTPUT_CAMERA)

    """ get_file """        
    def get_file(self):
        pass
        
    """ run """        
    def run(self, file):
        self.logger.info("run w Camera")  # inform the user

        now = datetime.now()  # current date and time
        date_time = now.strftime("%Y%m%d_%H%M%S")  # put to string nice format
        fname = conf.CAMERA_FOLDER + date_time + ".jpg"  # build the folder / file name
        
        cam = cv2.VideoCapture(0)  # capture
        s, img = cam.read()  # read
        if s:  # all good
            # WINDOW_NAME = 'Pic pic'
            # cv2.startWindowThread()
            # cv2.imshow(WINDOW_NAME,img)
            # cv2.waitKey(0) 
            # cv2.destroyAllWindows()
            # cv2.waitKey(1) 
            cv2.imwrite(fname, img)  # write

        else: 
            self.logger.info("no image detected")  # inform the user
        



# --------------------------
# Controllers folder
# Class definition
#
# Action : action management
# --------------------------

""" imports: logging, etc """
from biaspeech.utils.config import Config
import logging
import logging.config

""" imports: MVC """
from biaspeech.utils.text2Speech import Text2Speech

conf = Config()  # config


class Action():
    
    def __init__(self):

       """ util objects """
       self.logger = logging.getLogger(__name__)  # logger
       logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)
       
    """ button_man """ 
    def button_man(self): 
        conf.VOICE_MACOS = "com.apple.speech.synthesis.voice.thomas"  # man voice
        conf.VOICE_RASPBERRY = "french"  # man voice

        conf.write("text2Speech", "voice_macos", "com.apple.speech.synthesis.voice.thomas")
        conf.write("text2Speech", "voice_raspberry", "french")  

        self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech    
        self.text2Speech.do_speech(conf.MAN)  # inform the user
        
    """ button_woman """ 
    def button_woman(self): 
        conf.VOICE_MACOS = "com.apple.speech.synthesis.voice.amelie"  # woman voice
        conf.VOICE_RASPBERRY = "french+f3"  # mwoan voice

        conf.write("text2Speech", "voice_macos", "com.apple.speech.synthesis.voice.amelie")
        conf.write("text2Speech", "voice_raspberry", "french+f3")
    
        self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech    
        self.text2Speech.do_speech(conf.WOMAN)  # inform the user

    """ button_speed_m """ 
    def button_speed_m(self): 
        conf.RATE = conf.RATE - 10  # rate
        conf.write("text2Speech", "rate", str(conf.RATE))

        self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech    
        self.text2Speech.do_speech(str(conf.RATE))  # inform the user
        
    """ button_speed_p """ 
    def button_speed_p(self): 
        conf.RATE = conf.RATE + 10  # rate
        conf.write("text2Speech", "rate", str(conf.RATE))

        self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech    
        self.text2Speech.do_speech(str(conf.RATE))  # inform the user
                                
    """ button_help """ 
    def button_help(self): 
        self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech    
        self.text2Speech.do_speech(conf.HELP_APP)  # inform the user
        



# --------------------------
# Utils folder
# Class definition
#
# Config : constants
# --------------------------

""" imports: logging and config """
import os
import pathlib
from pathlib import Path
import configparser
        
""" config """
path = Path(__file__)
ROOT_DIR = path.parent.absolute()
config_path = os.path.join(ROOT_DIR, "config.cfg")
config = configparser.ConfigParser()
config.read(config_path)

class Config:

    """ constructor """
    def __init__(self): 
        parent = str(pathlib.Path(__file__).parent.parent.resolve()) # parent path 

        # read the arduino section of the config
        self.ARDUINO_INPUT_B = config.get("arduino", "input_b") # prompt text
        self.ARDUINO_FOLDER = parent + config.get("arduino", "folder") # folder to store ino file

        self.ARDUINO_PROGRAM = config.get("arduino", "program") # arduino program
        self.ARDUINO_BOARD = config.get("arduino", "board") # arduino board
        self.ARDUINO_PORT = config.get("arduino", "port") # arduino port
        self.ARDUINO_ACTIVE = config.get("arduino", "active") # arduino active

        # read the bia section of the config
        self.OUTPUT_ALEXA = config.get("bia", "output_alexa") # output by type
        self.OUTPUT_ARDUINO = config.get("bia", "output_arduino") # output by type
        self.OUTPUT_CAMERA = config.get("bia", "output_camera") # output by type
        self.OUTPUT_DISPLAY = config.get("bia", "output_display") # output by type
        self.OUTPUT_WEB = config.get("bia", "output_web") # output by type
        self.OUTPUT_RADIO = config.get("bia", "output_radio") # output by type
        self.OUTPUT_TV = config.get("bia", "output_tv") # output by type
        self.OUTPUT_YOUTUBE = config.get("bia", "output_youtube") # output by type
        self.OUTPUT_NETFLIX = config.get("bia", "output_netflix") # output by type
        self.OUTPUT_PYTHON = config.get("bia", "output_python") # output by type
        self.OUTPUT_MACRO = config.get("bia", "output_macro") # output by type
        
        # read the python section of the config
        self.CAMERA_FOLDER = parent + config.get("camera", "folder") # folder                  

        # read the error section of the config
        self.ERR_A = config.get("error", "err_a") # language
        self.ERR_B = config.get("error", "err_b") # language
        self.ERR_C = config.get("error", "err_c") # language
        self.ERR_D = config.get("error", "err_d") # language
                        
        # read the logging section of the config
        self.LOGLEVEL = config.get("logging", "level") # logging level 

        # read the main section of the config        
        self.OS = os.environ['OS'] # OS : macos or raspberry
        
        self.UI = config.get("main", "ui") # UI : keyboard or voice 
        self.MESSAGE = config.get("main", "message") # message 
        self.NOPROBLEM = config.get("main", "noproblem") # no problem 
        self.PROBLEM = config.get("main", "problem") # problem 
        self.VERSION = config.get("main", "version") # version 
        self.HELP = config.get("main", "help") # help 
        self.HELP_APP = config.get("main", "help_app") # help  app
        self.MAN = config.get("main", "man") # man
        self.WOMAN = config.get("main", "woman") # woman
        self.BACKGROUND = config.get("main", "background") # background
        self.WIDTH = int(config.get("main", "width")) # width
        self.HEIGHT = int(config.get("main", "height")) # height
                                                
        # read the openAI section of the config
        self.INPUT_MAX = config.get("openAI", "input_max") # input max
        
        # read the prompt section of the config
        self.LANG_TO = config.get("prompt", "lang_to") # pivot lang we convert to
        self.POSITIVE = config.get("prompt", "positive") # keyword for nltk
        self.NEGATIVE = config.get("prompt", "negative") # keyword for nltk
        self.NEUTRAL = config.get("prompt", "neutral") # keyword for nltk
        self.WEIGHT_AVG = config.getint("prompt", "weight_avg") # to trigger he type of prompt with the weight. Under this limit a weight will not trigger a type
        self.WEIGHT_MIN = config.getint("prompt", "weight_min") # to trigger a cache. Under this limit a mom or grandma with this weight will not be considered 
 
                
        # read the python section of the config
        self.PYTHON_INPUT_B = config.get("python", "input_b") # prompt text  
        self.PYTHON_INPUT_C = config.get("python", "input_c") # prompt text  
        self.PYTHON_FOLDER = parent + config.get("python", "folder") # folder to store the py file                 

        # read the speech2Text section of the config
        self.LANGUAGE = config.get("speech2Text", "language") # language
        self.RECORDING = parent + config.get("speech2Text", "recording") # recording
        self.RECORD = config.get("speech2Text", "record") + " " + self.RECORDING # recording
        self.ADJUST = config.getfloat("speech2Text", "adjust") # adjust
        self.TIMEOUT = config.getfloat("speech2Text", "timeout") # timeout
                                
        # read the sqlite section of the config
        self.DATABASE = parent + config.get("sqlite", "database") # database file 
                
        # read the text2Speech section of the config
        self.LANGUAGE = config.get("text2Speech", "language") # language 
        self.RATE = config.getint("text2Speech", "rate") # rate
        self.VOLUME = config.getfloat("text2Speech", "volume") # volume
        self.VOICE_MACOS = config.get("text2Speech", "voice_macos") # voice (3=Amelie, 38=Thomas)                                     
        self.VOICE_RASPBERRY = config.get("text2Speech", "voice_raspberry") # voice                               
        
        # read the web section of the config
        self.NETFLIX = config.get("web", "netflix") # netflix 
        self.RADIO = config.get("web", "radio") # radio       
        self.TV = config.get("web", "tv") # radio      
        self.YOUTUBE = config.get("web", "youtube") # youtube  
                        
        # other constants, not read in the config file
        self.WORDS = (["alexa"],
            ["bia", "robot"],
            ["camera"],
            ["display"],
            ["web", "internet"],
            ["radio"],
            ["tv"],
            ["youtube"],
            ["netflix"],
            ["python"],
            ["macro"],
            ["what", "why", "when", "where", "name", "is", "how", "do", "does", "which", "are", "could", "would", 
             "should", "has", "have", "whom", "whose", "don't"]) # key words to identify the prompt type

        self.TYPES = ("alexa", "arduino", "camera", "display", "web", "radio", "tv", "youtube", "netflix", "python", "macro", 
            "question","positive", "negative", "neutral") # prompt types
        
        self.CACHED = ("arduino", "python", "macro", "web", "radio", "tv", "youtube", "netflix", "question","positive", "negative", "neutral") # cached types, w WEIGHT_MIN
        
        self.FORCE_CACHED = ("positive", "negative", "neutral", "arduino") # cached types, whatever WEIGHT_MIN
        
        self.BACKPROPAGATE = ("positive", "negative", "neutral") # backpropagation types

        self.ARDUINO_INPUT_A = """ I have those examples :
  Otto.walk(2,1000,1); //2 steps, "TIME". IF HIGHER THE VALUE THEN SLOWER (from 600 to 1400), 1 FORWARD
  Otto.walk(2,1000,-1); //2 steps, T, -1 BACKWARD 
  Otto.turn(2,1000,1);//3 steps turning LEFT
  Otto._tone(10, 3, 1);
  Otto.bendTones (100, 200, 1.04, 10, 10);
    Otto.home();
    delay(100);  
  Otto.turn(2,1000,-1);//3 steps turning RIGHT 
  Otto.bend (1,500,1); //usually steps =1, T=2000
  Otto.bend (1,2000,-1);     
  Otto.shakeLeg (1,1500, 1);
    Otto.home();
    delay(100);
  Otto.shakeLeg (1,2000,-1);
  Otto.moonwalker(3, 1000, 25,1); //LEFT
  Otto.moonwalker(3, 1000, 25,-1); //RIGHT  
  Otto.crusaito(2, 1000, 20,1);
  Otto.crusaito(2, 1000, 20,-1);
    delay(100); 
  Otto.flapping(2, 1000, 20,1);
  Otto.flapping(2, 1000, 20,-1);
    delay(100);        
  Otto.swing(2, 1000, 20);
  Otto.tiptoeSwing(2, 1000, 20);
  Otto.jitter(2, 1000, 20); //(small T)
  Otto.updown(2, 1500, 20);  // 20 = H "HEIGHT of movement"T 
  Otto.ascendingTurn(2, 1000, 50);
  Otto.jump(1,500); // It doesn't really jumpl ;P
  Otto.home();
     delay(100); 
  Otto.sing(S_cuddly);
  Otto.sing(S_OhOoh);
  Otto.sing(S_OhOoh2);
  Otto.sing(S_surprise);
  Otto.sing(S_buttonPushed);       
  Otto.sing(S_mode1);        
  Otto.sing(S_mode2);         
  Otto.sing(S_mode3);  
  Otto.sing(S_sleeping);
  Otto.sing(S_fart1);
  Otto.sing(S_fart2);
  Otto.sing(S_fart3);
  Otto.sing(S_happy);
  Otto.sing(S_happy_short);                   
  Otto.sing(S_superHappy);   
  Otto.sing(S_sad);               
  Otto.sing(S_confused);
  Otto.sing(S_disconnection);
    delay(100);  
  Otto.playGesture(OttoHappy);
  Otto.playGesture(OttoSuperHappy);
  Otto.playGesture(OttoSad);
  Otto.playGesture(OttoVictory); 
  Otto.playGesture(OttoAngry); 
  Otto.playGesture(OttoSleeping);
  Otto.playGesture(OttoFretful);
  Otto.playGesture(OttoLove);
  Otto.playGesture(OttoConfused);        
  Otto.playGesture(OttoFart);
  Otto.playGesture(OttoWave);
  Otto.playGesture(OttoMagic);
  Otto.playGesture(OttoFail); 
  
  Start the code with :
  #include <Otto.h>
  Otto Otto;

  #define LeftLeg 2 // left leg pin, servo[0]
  #define RightLeg 3 // right leg pin, servo[1]
  #define LeftFoot 4 // left foot pin, servo[2]
  #define RightFoot 5 // right foot pin, servo[3]
  #define Buzzer 13 //buzzer pin

  void setup() {
    Otto.init(LeftLeg, RightLeg, LeftFoot, RightFoot, true, Buzzer);
    Otto.home();
  }
  
  please write a full code with the setup and loop methods to make the robot """

        self.PYTHON_INPUT_A = """# ** Polymorph code generated by openai :
# --------------------------
# Python folder
# Class definition
#
# Test : test the python polymorph code. In the program the 
#         code is generated by openai and stored in the python folder
# --------------------------

from controllers.base import Controller
import logging
import logging.config

from controllers.base import Controller

controller = Controller()

"""

    """ write """
    def write(self, section, option, value):
        config.set(section, option, value) # set the vqlue
        with open(config_path, 'w') as configfile: # write the file
            config.write(configfile)

             
        



# --------------------------
# Models folder
# Class definition
#
# AI : define the artificial intelligence model
# --------------------------

""" imports: logging, config, etc """
import openai
#import os
from biaspeech.utils.config import Config
import logging
import logging.config

""" objects: logging, config, etc """
MODEL = "gpt-3.5-turbo"
conf = Config() # config
     
class AI:    
    
    """ constructor """
    def __init__(self): 
        """ util objects """
        self.logger = logging.getLogger(__name__) # logger
        
    """ ask something with a limit """        
    def get_output(self, input):
        input = input + ' ' + conf.INPUT_MAX
        try:          
            response = openai.chat.completions.create(
                model=MODEL,
                messages=[{"role": "user", "content": input}]
            )
            output = response.choices[0].message.content.strip() # format the output
            return output

        except Exception as err:
            print(f"Unexpected {err=}, {type(err)=}")
            raise

    """ ask something without a limit """        
    def get_output_nolimit(self, input):
        try:          
            response = openai.chat.completions.create(
                model=MODEL,
                messages=[{"role": "user", "content": input}]
            )
            output = response.choices[0].message.content.strip() # format the output
            return output

        except Exception as err:
            print(f"Unexpected {err=}, {type(err)=}")
            raise




# --------------------------
# Utils folder
#
# Debug, lines to paste in the interpreter
# --------------------------

""" text to speech """
import pyttsx3
engine = pyttsx3.init()
#engine.setProperty('voice', 'french+f3') #my preference
engine.setProperty('voice', 'french')
engine.say("salut beau gosse") # say some
engine.runAndWait() # go
engine.stop() # stop
            
""" speech to text """
import speech_recognition as sr                                       
r = sr.Recognizer()                                                   
audio = sr.AudioFile("recording.wav")
with audio as source:
    audio = r.record(source)                  
    text = r.recognize_google(audio, language = 'fr_FR')
    
    print(format(text)) # format

import speech_recognition as sr 
recognizer = sr.Recognizer() # initialize the recognizer    
r = sr.Recognizer()                                                                                   
#with sr.Microphone(device_index = 3) as source: # set the deviceIndex         
with sr.Microphone(3) as source: # set the deviceIndex                                                              
    #recognizer.adjust_for_ambient_noise(source, duration=1) # w adjust duration
    recorded_audio = recognizer.listen(source) # listen
    text = recognizer.recognize_google(recorded_audio, language="fr_FR") # w language
    print(format(text)) # format

""" translate """
from translate import Translator
from langdetect import detect
from langdetect import DetectorFactory

text="quelle est la capitale de la france"
translator= Translator(from_lang="fr", to_lang="en")
result = translator.translate(text)
print(result)

result = detect(text)
print(result)

""" shell """
import os
stream = os.popen("arecord --format=S16_LE --duration=5 --rate=48000 ./data/recording.wav")



# --------------------------
# Models folder
# Class definition
#
# Python : define the Python (Block) model
# --------------------------

""" imports: logging and config """
import logging
import logging.config
from biaspeech.utils.config import Config
from datetime import datetime
import os

""" objects:  """
conf = Config()  # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
from biaspeech.models.blocks import block
from biaspeech.utils.text2Speech import Text2Speech


class Python(block.Block):
    
    """ constructor """
    def __init__(self, prompt, ai): 
        block.Block.__init__(self, prompt, ai)

    """ get_output """        
    def get_output(self): 
        return(conf.OUTPUT_PYTHON)
 
    """ get_file """        
    def get_file(self):
        self.logger.info("get_file")  # inform the user     
        
        search = self.prompt.mia.input.replace("python", "")  # search
        input_en = conf.PYTHON_INPUT_B + search + conf.PYTHON_INPUT_C  # build the inout for OpenAI
        
        code_raw = self.ai.get_output_nolimit(input_en)  # ask OpenAI an arduino code
        
        code_raw = code_raw.replace("==", "in")  # replace
        code_raw = code_raw.replace("!=", "not in")  # replace
        
        code = conf.PYTHON_INPUT_A + code_raw
        
        now = datetime.now()  # current date and time
        date_time = now.strftime("%Y%m%d_%H%M%S")  # put to string nice format
        fname = conf.PYTHON_FOLDER + date_time + ".py"  # build the folder / file name         
        
        file = open(fname, 'a')  # open up the file
        file.write(code)  # write
        file.close()  # close
        
        return(fname)
               
    """ run """        
    def run(self, file):
        try: 
            print(file)
            self.logger.info("python code generated")  # inform the user    
            os.popen("python3 " + file)
        
        except Exception as err:
            self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech    
            self.text2Speech.do_speech(conf.PROBLEM)  # inform the user
            self.logger.warning(f"conf.ERR_D {err=}, {type(err)=}")  # inform the user
      



# --------------------------
# Utils folder
# Class definition
#
# Paraphrase : paraphrase
# --------------------------

""" imports: logging, parrot """
import logging
import logging.config
from biaspeech.utils.config import Config
#from parrot import Parrot
import parrot

""" objects:  """
conf = Config() # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
from biaspeech.models.database.grandma import Grandma

class Paraphrase:

    """ constructor """
    def __init__(self): 
        
        """ util objects """
        self.conf = Config() # config
        self.logger = logging.getLogger(__name__) # logger

        """ attributes """        
        self.parrot = Parrot()
        self.grandma = Grandma(0, "", "", "", "")  
        
    """ set_paraphrase """
    def set_paraphrase(self):
        phrases = self.grandma.get_orphan()
                
        for phrase in phrases:
           paraphrases = self.parrot.augment(input_phrase=phrase[0])
           self.logger.info("searching paraphrase for : " + phrase[0]) # inform the user   

           if paraphrases:
               for paraphrase in paraphrases:
                   self.logger.info(paraphrase) # inform the user   
                   
                   self.grandma = Grandma(0, phrase[0], paraphrase[0], paraphrase[1], "") # new line in grandma
                   self.grandma.save()  



# --------------------------
# Models folder
# Class definition
#
# Block : define the Block model
# --------------------------

""" imports: logging and config """
import logging
import logging.config
from biaspeech.utils.config import Config

""" objects:  """
conf = Config()  # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
from biaspeech.models.speech import prompt
from biaspeech.models.openAI import ai


class Block:
    
    """ constructor """
    def __init__(self, prompt, ai): 
        
        self.logger = logging.getLogger(__name__)  # logger
        self.prompt = prompt  # prompt
        self.ai = ai  # ai

    """ get_output """        
    def get_output(self):
        pass

    """ get_file """        
    def get_file(self):
        pass
                    
    """ run """        
    def run(self, file):
        pass



# --------------------------
# Views folder
# Class definition
#
# App : define the base view, gui
# --------------------------

""" imports: logging, config, tkinter, threading, etc """
from biaspeech.utils.config import Config
from tkinter import ttk
from pathlib import Path
from tkinter import *
from pathlib import Path
import os
import logging
import logging.config
import tkinter as tk
import customtkinter
from PIL import Image, ImageTk

""" imports: MVC """
from biaspeech.utils.speech2Text import Speech2Text
from biaspeech.utils.text2Speech import Text2Speech
from biaspeech.controllers.asynch import Async
from biaspeech.controllers.action import Action

""" objects: logging and config """
path = Path(__file__)
ROOT_DIR = path.parent.absolute()
conf = Config() # config
        
class App(tk.Tk):
    def __init__(self):
        super().__init__()

        """ util objects """
        self.logger = logging.getLogger(__name__) # logger
       
        """ init """
        self.title(conf.MESSAGE) # title
        self.action = Action() # action
  
        self.gui_canvas() # gui, canvas
        self.gui_images() # gui, images
        self.gui_buttons() # gui, buttons
        self.gui_texts() # gui, texts

    """ image_push """ 
    def image_push(self, event):  
        self.button_push()            
          
    """ button_push """ 
    def button_push(self):    
        old_text_in = "" # init
        self.speech2Text = Speech2Text(conf.LANGUAGE) # speech2Text  
        self.speech2Text.do_listen() # start listening
        self.text_in = self.speech2Text.get_input().lower() # STT input 
        
        if self.text_in != old_text_in and self.text_in != "": # something changed
            thread = Async(self.text_in) # asynch object
            thread.start() # start thread
            self.monitor(thread) # monitor
                    
            old_text_in = self.text_in # update
                
    """ monitor """ 
    def monitor(self, thread):
        if thread.is_alive():
            self.after(100, lambda: self.monitor(thread)) # check the thread every 100ms
            self.canvas.itemconfig(self.prompt, text='>> ' + self.text_in + '\n>> ...')
                    
        else:
            conf = Config() # config reload
            self.canvas.itemconfig(self.prompt, text='>> ' + self.text_in + '\n>> ' + thread.output)
            self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY) # object for text2speech            
            self.text2Speech.do_speech(thread.output) # inform the user
        
    """ gui_canvas """
    def gui_canvas(self): # gui, canvas. Thanks to https://github.com/ParthJadhav/Tkinter-Designer
        geometry = str(conf.WIDTH) + 'x' + str(conf.HEIGHT)
        self.geometry(geometry)
        self.configure(bg = conf.BACKGROUND) # background
        
        self.canvas = Canvas( # canvas
            self,
            bg = conf.BACKGROUND,
            height = 519 * conf.HEIGHT / 519,
            width = 862 * conf.WIDTH / 862,
            bd = 0,
            highlightthickness = 0,
            relief = "ridge"
        )
        
        self.canvas.place(x = 0, y = 0)

    def gui_images(self): # gui, images
        file_path = os.path.join(ROOT_DIR, r'frame0/robot.png') # robot image
        image_robot = PhotoImage(file = file_path) 

        one = PhotoImage(file=file_path)
        self.one = one  # to prevent the image garbage collected.
        id_image_robot = self.canvas.create_image((490 * conf.HEIGHT / 519, 0), image=one, anchor='nw')
        self.canvas.tag_bind(id_image_robot, '<Button-1>', self.image_push)

    def gui_buttons(self): # gui, buttons
        custom_font = ("Roboto Bold", 25 * -1)
        button_push = customtkinter.CTkButton(master=self, text="PUSH TO TALK", font=custom_font, width=400 * conf.WIDTH / 862, height=150 * conf.HEIGHT / 519, compound="top", command=self.button_push) # button
        button_push.pack(padx=20 * conf.WIDTH / 862, pady=20 * conf.HEIGHT / 519) # push
        button_push.configure(hover_color="orange")        

        self.canvas_widget = self.canvas.create_window(680 * conf.WIDTH / 862, 50 * conf.HEIGHT / 519, window=button_push) # button

        file_path = os.path.join(ROOT_DIR, r'frame0/man.jpeg') # robot image
        image = customtkinter.CTkImage(Image.open(file_path), size=(50 * conf.WIDTH / 862, 50 * conf.HEIGHT / 519))
        button_man = customtkinter.CTkButton(master=self, text="", width=50 * conf.WIDTH / 862, height=50 * conf.HEIGHT / 519, compound="top", image=image, command=self.action.button_man) # button
        button_man.pack(pady=50 * conf.HEIGHT / 519, padx=50 * conf.WIDTH / 862) # man 
        self.canvas_widget = self.canvas.create_window(150 * conf.WIDTH / 862, 485 * conf.HEIGHT / 519, window=button_man) # button

        file_path = os.path.join(ROOT_DIR, r'frame0/woman.jpg') # robot image
        image = customtkinter.CTkImage(Image.open(file_path), size=(50 * conf.WIDTH / 862, 50 * conf.HEIGHT / 519))
        button_woman = customtkinter.CTkButton(master=self, text="", width=50 * conf.WIDTH / 862, height=50 * conf.HEIGHT / 519, compound="top", image=image, command=self.action.button_woman) # button
        button_woman.pack(pady=50 * conf.HEIGHT / 519, padx=50 * conf.WIDTH / 862) # man 
        self.canvas_widget = self.canvas.create_window(235 * conf.WIDTH / 862, 485 * conf.HEIGHT / 519, window=button_woman) # button
        
        button_speed_m = customtkinter.CTkButton(master=self, text="--", width=50 * conf.WIDTH / 862, height=50 * conf.HEIGHT / 519, compound="top", command=self.action.button_speed_m) # button
        button_speed_m.pack(pady=50 * conf.HEIGHT / 519, padx=50 * conf.WIDTH / 862) # rate   
        self.canvas_widget = self.canvas.create_window(320 * conf.WIDTH / 862, 485 * conf.HEIGHT / 519, window=button_speed_m) # button
                    
        button_speed_p = customtkinter.CTkButton(master=self, text="++", width=50 * conf.WIDTH / 862, height=50 * conf.HEIGHT / 519, compound="top", command=self.action.button_speed_p) # button
        button_speed_p.pack(pady=50 * conf.HEIGHT / 519, padx=50 * conf.WIDTH / 862) # rate   
        self.canvas_widget = self.canvas.create_window(380 * conf.WIDTH / 862, 485 * conf.HEIGHT / 519, window=button_speed_p) # button
                                    
        button_help = customtkinter.CTkButton(master=self, text="?", width=50 * conf.WIDTH / 862, height=50 * conf.HEIGHT / 519, compound="top", command=self.action.button_help) # button
        button_help.pack(pady=50 * conf.HEIGHT / 519, padx=50 * conf.WIDTH / 862) # help
        self.canvas_widget = self.canvas.create_window(440 * conf.WIDTH / 862, 485 * conf.HEIGHT / 519, window=button_help) # button

    def gui_texts(self): # gui, texts
        self.canvas.create_text( # text main
            16.0 * conf.WIDTH / 862,
            15.0 * conf.HEIGHT / 519,
            anchor="nw",
            text="Hey I am BIA :-)",
            fill="#FCFCFC",
            font=("Roboto Bold", 28 * -1)
        )
        
        self.canvas.create_rectangle(
            24.0 * conf.WIDTH / 862,
            78.0 * conf.HEIGHT / 519,
            84.0 * conf.WIDTH / 862,
            83.0 * conf.HEIGHT / 519,
            fill="#FCFCFC",
            outline="")
        
        self.canvas.create_text( # text second
            22.0 * conf.WIDTH / 862,
            85.0 * conf.HEIGHT / 519,
            anchor="nw",
            text=" your AI robot",
            fill="#FFFFFF",
            font=("Roboto Bold", 20 * -1)
        )
        
        self.canvas.create_text( # text prompt
            24.0 * conf.WIDTH / 862,
            145.0 * conf.HEIGHT / 519,
            width=690 * conf.WIDTH / 862,
            anchor="nw",
            text=">> yes? oui? wie kann ich dir helfen? como puedo ayudarte? 我怎么帮你 ",
            fill="#FFFFFF",
            font=("Roboto Bold", 18 * -1)
        )
        
        self.prompt = self.canvas.create_text( # text prompt second
                        24.0 * conf.WIDTH / 862,
                        220.0 * conf.HEIGHT / 519,
                        width=690 * conf.HEIGHT / 519,
                        anchor="nw",
                        text=">> ...",
                        fill="#FFFFFF",
                        font=("Roboto Bold", 18 * -1))    
                 
        self.canvas.create_text( # text prompt third
            20.0 * conf.WIDTH / 862,
            490.0 * conf.HEIGHT / 519,
            anchor="nw",
            text=conf.VERSION,
            fill="#FFFFFF",
            font=("Roboto Bold", 6 * -1)
        )  
        
        self.canvas.pack()
        #window.resizable(False, False)




# --------------------------
# Models folder
# Class definition
#
# Alexa : define the Alexa (Block) model
# --------------------------

""" imports: logging and config """
import logging
import logging.config
from biaspeech.utils.config import Config

""" objects:  """
conf = Config()  # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
from biaspeech.models.blocks import block


class Alexa(block.Block):
    
    """ constructor """
    def __init__(self, prompt, ai): 
        block.Block.__init__(self, prompt, ai)
        
    """ get_output """        
    def get_output(self):
        return(self.conf.OUTPUT_ALEXA)

    """ get_file """        
    def get_file(self):
        pass
        
    """ run """        
    def run(self, file):
        self.logger.info("run w Alexa")  # inform the user 
        



# --------------------------
# Models folder
# Class definition
#
# Arduino : define the Arduino (Block) model
# --------------------------

""" imports: logging, config, etc """
import logging
import logging.config
from biaspeech.utils.config import Config
from datetime import datetime
import pyduinocli
import os

""" objects:  """
conf = Config()  # config
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=conf.LOGLEVEL)

""" MVC """
from biaspeech.models.blocks import block
from biaspeech.utils.text2Speech import Text2Speech

class Arduino(block.Block):
    
    """ constructor """
    def __init__(self, prompt, ai): 
        block.Block.__init__(self, prompt, ai)

    """ get_output """        
    def get_output(self): 
        return(conf.OUTPUT_ARDUINO)
 
    """ get_file """        
    def get_file(self):
        self.logger.info("get_file")  # inform the user     
        
        input_en = conf.ARDUINO_INPUT_A + self.prompt.mia.input_en + conf.ARDUINO_INPUT_B  # build the inout for OpenAI
        code_raw = self.ai.get_output_nolimit(input_en)  # ask OpenAI an arduino code
        
        code = code_raw
        
        now = datetime.now()  # current date and time
        date_time = now.strftime("%Y%m%d_%H%M%S")  # put to string nice format
        folder = conf.ARDUINO_FOLDER + date_time
        fname = folder + "/" + date_time + ".ino"  # build the folder / file name        
        
        os.mkdir(folder)  # will save the file
        file = open(fname, 'a')  # open up the file
        file.write(code)  # write
        file.close()  # close
        
        return(folder)
               
    """ run """        
    def run(self, folder):
        try: 
            if conf.ARDUINO_ACTIVE == "1":  # arduino option on
                arduino = pyduinocli.Arduino(conf.ARDUINO_PROGRAM)  # program
                arduino.compile(fqbn=conf.ARDUINO_BOARD, sketch=folder)  # compile
                arduino.upload(fqbn=conf.ARDUINO_BOARD, sketch=folder, port=conf.ARDUINO_PORT)  # upload
                self.logger.info("arduino code uploaded")  # inform the user    
                            
            else:
                self.logger.info("arduino option is off")  # inform the user      
        
        except Exception as err:
            self.text2Speech = Text2Speech(conf.LANGUAGE, conf.RATE, conf.VOLUME, conf.VOICE_MACOS, conf.VOICE_RASPBERRY)  # object for text2speech    
            self.text2Speech.do_speech(conf.PROBLEM)  # inform the user
            self.logger.warning(f"conf.ERR_D {err=}, {type(err)=}")  # inform the user
